@@ -7,17 +7,28 @@ use Carp;
 use List::Util qw(sum min max);
 use Data::Dumper;
 
+use Devel::NYTProf::Util qw(
+    trace_level
+);
 use Devel::NYTProf::Constants qw(
     NYTP_SIi_FID NYTP_SIi_FIRST_LINE NYTP_SIi_LAST_LINE
     NYTP_SIi_CALL_COUNT NYTP_SIi_INCL_RTIME NYTP_SIi_EXCL_RTIME
     NYTP_SIi_SUB_NAME NYTP_SIi_PROFILE
     NYTP_SIi_REC_DEPTH NYTP_SIi_RECI_RTIME NYTP_SIi_CALLED_BY
+    NYTP_SIi_elements
 
+    NYTP_SCi_CALL_COUNT
     NYTP_SCi_INCL_RTIME NYTP_SCi_EXCL_RTIME NYTP_SCi_RECI_RTIME
-    NYTP_SCi_CALLING_SUB
+    NYTP_SCi_REC_DEPTH NYTP_SCi_CALLING_SUB
+    NYTP_SCi_elements
 );
 
-my $trace = (($ENV{NYTPROF}||'') =~ m/\b trace=(\d+) /x) && $1; # XXX a hack
+# extra constants for private elements
+use constant {
+    NYTP_SIi_meta            => NYTP_SIi_elements + 1,
+    NYTP_SIi_cache           => NYTP_SIi_elements + 2,
+};
+
 
 sub fid        { shift->[NYTP_SIi_FID] || 0 }
 
@@ -55,6 +66,13 @@ sub recur_max_depth { shift->[NYTP_SIi_REC_DEPTH] }
 
 sub recur_incl_time { shift->[NYTP_SIi_RECI_RTIME] }
 
+
+# general purpose hash - mainly a hack to help kill off Reader.pm
+sub meta      { shift->[NYTP_SIi_meta()] ||= {} }
+# general purpose cache
+sub cache     { shift->[NYTP_SIi_cache()] ||= {} }
+
+
 # { fid => { line => [ count, incl_time ] } }
 sub caller_fid_line_places {
     my ($self, $merge_evals) = @_;
@@ -96,6 +114,10 @@ sub is_opcode {
     return 0;
 }
 
+sub is_anon {
+    shift->subname =~ m/::__ANON__\b/;
+}
+
 sub kind {
     my $self = shift;
     return 'opcode' if $self->is_opcode;
@@ -136,14 +158,24 @@ sub _max {
 }
 
 
-sub alter_fileinfo {
+sub _alter_fileinfo {
     my ($self, $remove_fi, $new_fi) = @_;
     my $remove_fid = ($remove_fi) ? $remove_fi->fid : 0;
     my $new_fid    = (   $new_fi) ?    $new_fi->fid : 0;
 
-    warn sprintf "Altering fileinfo of %s from %d to %d\n",
-            $self->subname, $remove_fid, $new_fid
-        if $trace;
+    if ($self->fid == $remove_fid) {
+        $self->[NYTP_SIi_FID] = $new_fid;
+
+        $remove_fi->_remove_sub_defined($self) if $remove_fi;
+        $new_fi->_add_new_sub_defined($self) if $new_fi;
+    }
+}
+
+
+sub _alter_called_by_fileinfo {
+    my ($self, $remove_fi, $new_fi) = @_;
+    my $remove_fid = ($remove_fi) ? $remove_fi->fid : 0;
+    my $new_fid    = (   $new_fi) ?    $new_fi->fid : 0;
 
     # remove mentions of $remove_fid from called-by details
     # { fid => { line => [ count, incl, excl, ... ] } }
@@ -151,101 +183,126 @@ sub alter_fileinfo {
         my $cb = delete $called_by->{$remove_fid};
 
         if ($cb && $new_fid) {
-            if (my $new_cb = $called_by->{$new_fid}) {
-                # need to merge $cb into $new_cb
-                while ( my ($line, $cb_li) = each %$cb ) {
-                    my $dst_line_info = $new_cb->{$line} ||= [];
-                    _merge_in_caller_info($dst_line_info, $cb->{$line});
-                }
-            }
-            else {
-                $called_by->{$new_fid} = $cb;
+            my $new_cb = $called_by->{$new_fid} ||= {};
+
+            warn sprintf "_alter_called_by_fileinfo: %s from fid %d to fid %d\n",
+                    $self->subname, $remove_fid, $new_fid
+                if trace_level();
+
+            # merge $cb into $new_cb
+            while ( my ($line, $cb_li) = each %$cb ) {
+                my $dst_line_info = $new_cb->{$line} ||= [];
+                _merge_in_caller_info($dst_line_info, delete $cb->{$line},
+                    tag => "$line:".$self->subname,
+                );
             }
+
         }
     }
 
-    if ($self->fid == $remove_fid) {
-        $self->[NYTP_SIi_FID] = $new_fid;
-    }
-
-    if ($remove_fi and $remove_fi->_remove_sub_defined($self)) {
-        $new_fi->_add_new_sub_defined($self);
-    }
 }
 
 
+
+
 # merge details of another sub into this one
 # there are very few cases where this is sane thing to do
 # it's meant for merging things like anon-subs in evals
 # e.g., "PPI::Node::__ANON__[(eval 286)[PPI/Node.pm:642]:4]"
 sub merge_in {
-    my $self = shift;
-    my $new = shift;
+    my ($self, $donor, %opts) = @_;
+    my $self_subname  = $self->subname;
+    my $donor_subname = $donor->subname;
+
+    warn sprintf "Merging sub %s into %s (%s)\n",
+            $donor_subname, $self_subname, join(" ", %opts)
+        if trace_level();
 
     # see also "case NYTP_TAG_SUB_CALLERS:" in load_profile_data_from_stream()
+    push @{ $self->meta->{merged_sub_names} }, $donor->subname;
 
-    $self->[NYTP_SIi_FIRST_LINE]  = _min($self->[NYTP_SIi_FIRST_LINE], $new->[NYTP_SIi_FIRST_LINE]);
-    $self->[NYTP_SIi_LAST_LINE]   = _max($self->[NYTP_SIi_LAST_LINE],  $new->[NYTP_SIi_LAST_LINE]);
-
-    $self->[NYTP_SIi_CALL_COUNT] += $new->[NYTP_SIi_CALL_COUNT];
-    $self->[NYTP_SIi_INCL_RTIME] += $new->[NYTP_SIi_INCL_RTIME];
-    $self->[NYTP_SIi_EXCL_RTIME] += $new->[NYTP_SIi_EXCL_RTIME];
-    $self->[NYTP_SIi_SUB_NAME]    = [ $self->[NYTP_SIi_SUB_NAME] ]
-        if not ref $self->[NYTP_SIi_SUB_NAME];
-    push @{$self->[NYTP_SIi_SUB_NAME]}, $new->[NYTP_SIi_SUB_NAME];
-    $self->[NYTP_SIi_REC_DEPTH]   = max($self->[NYTP_SIi_REC_DEPTH], $new->[NYTP_SIi_REC_DEPTH]);
+    $self->[NYTP_SIi_FIRST_LINE]  = _min($self->[NYTP_SIi_FIRST_LINE], $donor->[NYTP_SIi_FIRST_LINE]);
+    $self->[NYTP_SIi_LAST_LINE]   = _max($self->[NYTP_SIi_LAST_LINE],  $donor->[NYTP_SIi_LAST_LINE]);
+    $self->[NYTP_SIi_CALL_COUNT] += $donor->[NYTP_SIi_CALL_COUNT];
+    $self->[NYTP_SIi_INCL_RTIME] += $donor->[NYTP_SIi_INCL_RTIME];
+    $self->[NYTP_SIi_EXCL_RTIME] += $donor->[NYTP_SIi_EXCL_RTIME];
+    $self->[NYTP_SIi_REC_DEPTH]   = max($self->[NYTP_SIi_REC_DEPTH], $donor->[NYTP_SIi_REC_DEPTH]);
     # adding reci_rtime is correct only if one sub doesn't call the other
-    $self->[NYTP_SIi_RECI_RTIME] += $new->[NYTP_SIi_RECI_RTIME]; # XXX
+    $self->[NYTP_SIi_RECI_RTIME] += $donor->[NYTP_SIi_RECI_RTIME]; # XXX
 
     # { fid => { line => [ count, incl_time, ... ] } }
-    my $dst_called_by = $self->[NYTP_SIi_CALLED_BY] ||= {};
-    my $src_called_by = $new ->[NYTP_SIi_CALLED_BY] ||  {};
+    my $dst_called_by = $self ->[NYTP_SIi_CALLED_BY] ||= {};
+    my $src_called_by = $donor->[NYTP_SIi_CALLED_BY] ||  {};
 
-    my $subname = $self->subname(' and ');
+    $opts{opts} ||= "merge in $donor_subname";
 
     # iterate over src and merge into dst
     while (my ($fid, $src_line_hash) = each %$src_called_by) {
+
         my $dst_line_hash = $dst_called_by->{$fid};
-        if (!$dst_line_hash) {
-            $dst_called_by->{$fid} = $src_line_hash;
-            warn "renamed sub caller $self->[NYTP_SIi_SUB_NAME] into $subname\n" if $trace;
-            next;
-        }
-        warn "merged sub caller $self->[NYTP_SIi_SUB_NAME] into $subname\n" if $trace;
 
         # merge lines in %$src_line_hash into %$dst_line_hash
-        while (my ($line, $src_line_info) = each %$src_line_hash) {
+        for my $line (keys %$src_line_hash) {
             my $dst_line_info = $dst_line_hash->{$line} ||= [];
-            _merge_in_caller_info($dst_line_info, $src_line_info);
+            my $src_line_info = $src_line_hash->{$line};
+            delete $src_line_hash->{$line} unless $opts{src_keep};
+            _merge_in_caller_info($dst_line_info, $src_line_info, %opts);
         }
     }
+
     return;
 }
 
 
 sub _merge_in_caller_info {
-    my ($dst_line_info, $src_line_info) = @_;
+    my ($dst_line_info, $src_line_info, %opts) = @_;
+    my $tag = ($opts{tag}) ? " $opts{tag}" : "";
 
+    if (!@$src_line_info) {
+        carp sprintf "_merge_in_caller_info%s skipped (empty donor)", $tag
+            if trace_level();
+        return;
+    }
+
+    if (trace_level()) {
+        carp sprintf "_merge_in_caller_info%s merging from $src_line_info -> $dst_line_info:", $tag;
+        warn sprintf " . %s\n", fmt_sc($src_line_info);
+        warn sprintf " + %s\n", fmt_sc($dst_line_info);
+    }
     if (!@$dst_line_info) {
-        @$dst_line_info = @$src_line_info;
+        @$dst_line_info = (0) x NYTP_SCi_elements;
+        $dst_line_info->[NYTP_SCi_CALLING_SUB] = undef;
     }
-    else {
 
-        # merge @$src_line_info into @$dst_line_info
-        $dst_line_info->[$_] += $src_line_info->[$_] for (
-            NYTP_SCi_INCL_RTIME, NYTP_SCi_EXCL_RTIME,
-        );
-        # ug, we can't really combine recursive incl_time, but this is better than undef
-        $dst_line_info->[NYTP_SCi_RECI_RTIME] = max($dst_line_info->[NYTP_SCi_RECI_RTIME],
-                                                    $src_line_info->[NYTP_SCi_RECI_RTIME]);
+    # merge @$src_line_info into @$dst_line_info
+    $dst_line_info->[$_] += $src_line_info->[$_] for (
+        NYTP_SCi_CALL_COUNT, NYTP_SCi_INCL_RTIME, NYTP_SCi_EXCL_RTIME,
+    );
+    $dst_line_info->[NYTP_SCi_REC_DEPTH] = max($dst_line_info->[NYTP_SCi_REC_DEPTH],
+                                                $src_line_info->[NYTP_SCi_REC_DEPTH]);
+    # ug, we can't really combine recursive incl_time, but this is better than undef
+    $dst_line_info->[NYTP_SCi_RECI_RTIME] = max($dst_line_info->[NYTP_SCi_RECI_RTIME],
+                                                $src_line_info->[NYTP_SCi_RECI_RTIME]);
+
+    my $src_cs = $src_line_info->[NYTP_SCi_CALLING_SUB]|| {};
+    my $dst_cs = $dst_line_info->[NYTP_SCi_CALLING_SUB]||={};
+    $dst_cs->{$_} = $src_cs->{$_} for keys %$src_cs;
+
+    warn sprintf " = %s\n", fmt_sc($dst_line_info)
+        if trace_level();
 
-        my $src_cs = $src_line_info->[NYTP_SCi_CALLING_SUB]||={};
-        my $dst_cs = $dst_line_info->[NYTP_SCi_CALLING_SUB]||={};
-        $dst_cs->{$_} = $src_cs->{$_} for keys %$src_cs;
-    }
     return;
 }
 
+sub fmt_sc {
+    my ($sc) = @_;
+    return "(empty)" if !@$sc;
+    my $dst_cs = $sc->[NYTP_SCi_CALLING_SUB]||{};
+    my $by = join " & ", sort keys %$dst_cs;
+    sprintf "calls %d%s",
+        $sc->[NYTP_SCi_CALL_COUNT], ($by) ? ", by $by" : "";
+}
+
 
 sub caller_fids {
     my ($self, $merge_evals) = @_;
@@ -328,26 +385,36 @@ sub normalize_for_test {
 sub dump {
     my ($self, $separator, $fh, $path, $prefix) = @_;
 
+    my ($fid, $l1, $l2, $calls) = @{$self}[
+        NYTP_SIi_FID, NYTP_SIi_FIRST_LINE, NYTP_SIi_LAST_LINE, NYTP_SIi_CALL_COUNT
+    ];
     my @values = @{$self}[
-        NYTP_SIi_FID, NYTP_SIi_FIRST_LINE, NYTP_SIi_LAST_LINE,
-        NYTP_SIi_CALL_COUNT, NYTP_SIi_INCL_RTIME, NYTP_SIi_EXCL_RTIME,
+        NYTP_SIi_INCL_RTIME, NYTP_SIi_EXCL_RTIME,
         NYTP_SIi_REC_DEPTH, NYTP_SIi_RECI_RTIME
     ];
-    printf $fh "%s[ %s ]\n",
-        $prefix, join(" ", map { defined($_) ? $_ : 'undef' } @values);
+    printf $fh "%s[ %s:%s-%s calls %s times %s ]\n",
+        $prefix,
+        map({ defined($_) ? $_ : 'undef' } $fid, $l1, $l2, $calls),
+        join(" ", map { defined($_) ? $_ : 'undef' } @values);
 
     my @caller_places = $self->caller_places;
     for my $cp (@caller_places) {
         my ($fid, $line, $sc) = @$cp;
         my @sc = @$sc;
         $sc[NYTP_SCi_CALLING_SUB] = join "|", keys %{ $sc[NYTP_SCi_CALLING_SUB] };
-        printf $fh "%s%s%s%d%s%d%s[ %s ]\n",
+        printf $fh "%s%s%s%d:%d%s[ %s ]\n",
             $prefix,
             'called_by', $separator,
-            $fid,  $separator,
-            $line, $separator,
+            $fid, $line, $separator,
             join(" ", map { defined($_) ? $_ : 'undef' } @sc);
     }
+
+    # where a sub has had others merged into it, list them
+    my $merge_subs = $self->meta->{merged_sub_names} || [];
+    for my $ms (sort @$merge_subs) {
+        printf $fh "%s%s%s%s\n",
+            $prefix, 'merge_donor', $separator, $ms;
+    }
 }
 
 # vim:ts=8:sw=4:et
